// @(#)bitswapper.cpp	1.2 - 98/04/28
#include <string.h>

#include "bitswapper.h"

BitSwapper::BitSwapper (
	unsigned bs,
	unsigned ni,
	uint32 seed
) : rng(seed)
{
	niters = ni;	// yes, even if it is zero!

//--[determine blocksz]--
// keep increasing m until m>=bs or until m==MAXSWAPBYTES
// this will round bs upto the nearest power of two
//---
	unsigned m;
	for(m=1; m<MAXSWAPBYTES && m<bs; m<<=1)
		;
	blocksz = m;
}

#define canmask(M,Y,V) ((M[Y]&V) == 0)

// single iteration - swap two bits
void
BitSwapper::bbswap1 (
	uint8 *block,
	uint8 *rembits
)
{
// bigmask grabs numbers in the range 0<=x<blocksz*8
	unsigned bigmask = (blocksz<<3)-1;
	uint32 rnd = rng.lRand();

// r1 and r2 are the bit positions
	uint8 r1 = (rnd>>8) & bigmask;
	uint8 r2 = (rnd>>16) & bigmask;

// don't swap if positions are the same
	if(r1 != r2)
	{
	// y1 and y2 are byte numbers and will be in range 0 .. blocksz-1
		uint8 y1 = r1>>3;
		uint8 y2 = r2>>3;

	// v1 and v2 are 8bit masks
		uint8 v1 = 1<<(r1&7);
		uint8 v2 = 1<<(r2&7);

	// check whether we have tried these bits before
		if(canmask(rembits,y1,v1) && canmask(rembits,y2,v2) )
		{
		// x1 and x2 extract the bits from block
			uint8 x1 = block[y1]&v1;
			uint8 x2 = block[y2]&v2;

		// we can only swap if the bits are different
			if( (x1 == 0 && x2 != 0) || (x2 == 0 && x1 != 0) )
			{
				block[y1] ^= v1;
				block[y2] ^= v2;
			}
		
		// remember the bits
			rembits[y1] |= v1;
			rembits[y2] |= v2;
		}
	}
}

// bitswap a single block n times
void
BitSwapper::bbswapb (
	const uint8 *plain,
	uint8 *cipher
)
{
	uint8 rembits[MAXSWAPBYTES];

// empty the remembered bits
	memset(rembits, 0, blocksz);

// initialise the cipher with the plain
	memcpy(cipher, plain, blocksz);

// swap the bits of the cipher block
	for(unsigned i=0; i<niters; i++)
		bbswap1(cipher, rembits);
}

// bitswap a single part-block
unsigned
BitSwapper::part (
	const uint8 *plain,
	uint8 *cipher,
	unsigned partsz
)
{
// handle the last part-block
	unsigned c = 0;

	blocksz >>= 1;
	niters /= 2;

	while(partsz != 0 && blocksz != 0)
	{
		if( (partsz & blocksz) != 0)
		{
			bbswapb (plain+c, cipher+c);

			c += blocksz;
			partsz -= blocksz;
		}

		blocksz >>= 1;
		niters /= 2;
	}

	return c;
}

unsigned
BitSwapper::Swap (
	const uint8 *plain,
	uint8 *cipher,
	unsigned nbytes
)
{
	unsigned rval = blocksz;

	if(nbytes == 0)
		rval = 0;
	else if(nbytes < blocksz)
	{
	// since 'part' changes things, we need to save and restore
		unsigned save_blocksz = blocksz;
		unsigned save_niters = niters;

		rval = part (plain, cipher, nbytes);

		blocksz = save_blocksz;
		niters = save_niters;
	}
	else
		bbswapb (plain, cipher);	// rval = blocksz;

	return rval;
}

